"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const missing_route_params_error_1 = require("./missing-route-params-error");
const convertToString = (value) => {
    if (value instanceof Date)
        return encodeURIComponent(value.toISOString());
    return encodeURIComponent(value.toString());
};
class SimpleUrlBuilder {
    constructor(baseUrl = '') {
        this.baseUrl = baseUrl;
        this.routeParams = {};
        this.queryParams = {};
    }
    addQueryParam(key, value) {
        if (value == null)
            return this;
        if (Array.isArray(value))
            return this.addArrayValues(value, key);
        this.queryParams[key] = convertToString(value);
        return this;
    }
    addArrayValues(value, key) {
        value.forEach((item, index) => {
            if (item == null)
                return;
            this.queryParams[`${key}[${index}]`] = convertToString(item);
        });
        return this;
    }
    addRouteParam(key, value) {
        if (value == null || value === '')
            return this;
        this.routeParams[key] = convertToString(value);
        return this;
    }
    build() {
        const route = this.populateRouteParams();
        const queryParams = this.getQueryParams();
        if (route === '')
            return queryParams;
        if (queryParams == null || queryParams === '')
            return route;
        if (route.indexOf('?') === -1)
            return `${route}?${queryParams}`;
        return `${route}&${queryParams}`;
    }
    getQueryParams() {
        const keys = Object.keys(this.queryParams);
        return keys.reduce((acc, key) => {
            return acc.concat(`${key}=${this.queryParams[key]}`);
        }, []).join('&');
    }
    populateRouteParams() {
        const keys = Object.keys(this.routeParams);
        const route = keys.reduce((acc, key) => {
            return acc.replace(`{${key}}`, this.routeParams[key]);
        }, this.baseUrl);
        if (SimpleUrlBuilder.templateRegEx.test(route)) {
            throw new missing_route_params_error_1.MissingRouteParamsError(`Route still contains template params: ${route}`);
        }
        return route;
    }
}
exports.default = SimpleUrlBuilder;
SimpleUrlBuilder.templateRegEx = /\{[a-zA-z\d]+\}/;
//# sourceMappingURL=simple-url-builder.js.map